@@ -373,8 +373,12 @@ class BuildWaveCalib:
             # Internal mask for failed wv_calib analysis
             # TODO -- Allow for an option to re-attempt those previously flagged as BADWVCALIB?
             self.wvc_bpm = np.invert(mask == 0)
+            ## We want to keep the 'BOXSLIT', which mask value is 2. But we don't want to keep 'BOXSLIT'
+            ## with other bad flag (for which the mask value would be > 2)
+            #self.wvc_bpm = mask > 2
             self.wvc_bpm_init = self.wvc_bpm.copy()
             # Slitmask -- Grabs only unmasked, initial slits
+            #self.slitmask_science = self.slits.slit_img(initial=True, flexure=None, exclude_flag=['BOXSLIT'])
             self.slitmask_science = self.slits.slit_img(initial=True, flexure=None)
             # Resize
             self.shape_science = self.slitmask_science.shape
@@ -437,7 +441,9 @@ class BuildWaveCalib:
                                                     IDwaves=self.par['IDwaves'])
         elif method == 'holy-grail':
             # Sometimes works, sometimes fails
-            arcfitter = autoid.HolyGrail(arccen, par=self.par, ok_mask=ok_mask_idx, nonlinear_counts=self.nonlinear_counts)
+            arcfitter = autoid.HolyGrail(arccen, par=self.par, ok_mask=ok_mask_idx, 
+                                         nonlinear_counts=self.nonlinear_counts,
+                                         spectrograph=self.spectrograph.name)
             patt_dict, final_fit = arcfitter.get_results()
         elif method == 'identify':
             final_fit = {}
